/**
 * module.h: Module framework.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in doc/pod/gplicense.pod.
 *
 * $Id: module.h,v 1.5 2007/09/22 14:27:26 pippijn Exp $
 */

#ifndef ERMYTH_MODULE_H
#define ERMYTH_MODULE_H

#include <boost/foreach.hpp>

#include <vector>

#include <util/exception.h>
#include <ermyth/phoenix.h>

struct module
{
  char const * const name;
  bool const norestart;
  bool (*init) (module *m);
  void (*fini) (void);
  char const * const vendor;
  char const * const version;
  bool enabled;
  
  module (char const * const modname, bool modnorestart, bool (*modinit)(module *), void (*modfini)(),
          char const * const modvendor, char const * const modversion)
  : name (modname), norestart (modnorestart), init (modinit), fini (modfini),
    vendor (modvendor), version (modversion), enabled (false)
  {
  }
};

struct modules
{
  typedef std::vector<module *> list_type;

  static list_type const &list ()
  {
    return ::impl::phoenix<list_type>::instance ();
  }

  modules (impl::srcinf const &si, char const * const name, bool norestart, bool (*init)(module *), void (*fini)(),
           char const * const vendor, char const * const modversion);

  static bool provides (char const * const name);
  static fault::code enable (char const * const name);
  static fault::code disable (char const * const name);
  static module *find (char const * const name);
  static void cleanup ();

private:
  static list_type &modlist ()
  {
    return ::impl::phoenix<list_type>::instance ();
  }
};

#define REGISTER_MODULE(name, norestart, vendor) \
  static bool _modinit (module *m);              \
  static void _moddeinit (void);                 \
  static modules registered_module (SRCINF, name, norestart, _modinit, _moddeinit, vendor, rcsid)

// TODO: REGISTER_DEPENDENCY and dependency handling
#define REGISTER_DEPENDENCY(thismod, othermod)

#endif
