#ifndef ERMYTH_PHOENIX_H
#define ERMYTH_PHOENIX_H

#include <stdlib.h> // atexit()


namespace impl
{
  struct no_op_phoenix_initialiser
  {
    template<typename T>
    void operator () (T &)
    {
      return;
    }
  };

  template<typename BaseType, typename ContextType = BaseType, typename initialiserType = no_op_phoenix_initialiser>
  struct phoenix : public BaseType
  {
    typedef ContextType context_type;
    typedef BaseType base_type;
    typedef initialiserType initialiser_type;

    static base_type &instance ()
    {
      static this_type meyers;
      static bool donethat = false;
      
      if (this_type::m_destroyed)
        {
          // phoenix_CERR << "Phoenixing!" << std::endl;
          donethat = false;
          new (&meyers) this_type;
          atexit (this_type::do_atexit);
        }

      if (!donethat)
        {
          // phoenix_CERR << "initialising instance" << std::endl;
          donethat = true;
          initialiser_type () (meyers);
        }
      
      // phoenix_CERR << "instance() == " <<std::hex<<&meyers<<std::endl;
      return meyers;
    }

  private:
    typedef phoenix<base_type,context_type,initialiser_type> this_type;

    static bool m_destroyed;

    phoenix ()
    {
      // phoenix_CERR << "phoenix() @" << std::hex<< this << std::endl;
      m_destroyed = false;
    }

    virtual ~phoenix () throw ()
    {
      // phoenix_CERR << "~phoenix() @" << std::hex<< this << std::endl;
      m_destroyed = true;
    }

    // Destroys the shared object via a manual call to it's dtor.
    static void do_atexit()
    {
      if (m_destroyed)
        return;
      // phoenix_CERR << "::do_atexit() @ " << std::hex << &instance() << std::endl;
      static_cast<this_type &> (instance ()).~phoenix (); // will eventually trigger the BaseType dtor
    }
  };

  template<typename T, typename C, typename I>
  bool phoenix<T, C, I>::m_destroyed = false;
} // namespace

#endif // ERMYTH_PHOENIX_H
