/**
 * shstr.h: Shared string class.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in doc/pod/gplicense.pod.
 *
 * $Id: shstr.h,v 1.3 2007/09/16 18:54:42 pippijn Exp $
 */

#ifndef SHSTR_H
#define SHSTR_H

#include <string>
#include <vector>
#include <iostream>

#ifdef WANT_STRLEN
# define SHSTRLEN(str)  strlen (str)
#else
# define SHSTRLEN(str)  0
#endif

struct shentry
{
  shentry (char const * const str, const size_t hash)
  : m_refcnt (0), m_str (str), m_len (SHSTRLEN (str)), m_hash (hash)
  {
  }

  mutable int m_refcnt;
  char const * const m_str;
  size_t m_len;
  size_t m_hash;
};

class shstr
{
protected:
  static size_t m_cnt;

  static size_t m_bucketcnt;

  typedef std::vector<shentry *> pairs;

  static pairs *m_buckets;

  static shentry *prepare (char const * const str);
  static bool unref (char const * const str);
  static const shentry *get (char const * const str);

  static pairs::iterator find (char const * const str, size_t n);

  static size_t hash (char const * const str);

public:
  static char const * const insert (char const * const str, size_t *size);

  static void erase (char const * const str);

  static size_t bucketcnt ()
  {
    return m_bucketcnt;
  }

  static size_t refs (char const * const str);
  static size_t strcnt ();

  static size_t memused ();

  static void initbuckets (pairs *buckets)
  {
    m_buckets = buckets;
  }

  static void cleanup ()
  {
    delete[] m_buckets;
  }

public:
  typedef size_t size_type;

  shstr (char const * const str = "");
  shstr (const shstr &rhs);
  ~shstr ();

  char const * const c_str () const;
  size_type length () const;
  shstr &operator = (char const * const rhs);
  shstr &operator = (const shstr &rhs);
  bool operator == (const shstr &rhs) const;
  bool operator != (const shstr &rhs) const;

  friend std::ostream &operator << (std::ostream &os, const shstr &str)
  {
    os << str.m_str;
    return os;
  }

protected:
  char const *m_str;
  size_type m_len;

  void erase ()
  {
    erase (m_str);
    m_str = NULL;
  }

  void insert (char const * const newstr)
  {
    m_str = insert (newstr, &m_len);
  }
};

#endif // SHSTR_H
