/**
 * type_traits.h: Some templates that provide similar functionality as tr1::type_traits, but specialised for Ermyth.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in doc/pod/gplicense.pod.
 *
 * $Id: type_traits.h,v 1.5 2007/09/16 18:54:42 pippijn Exp $
 */

#ifndef ERMYTH_TYPE_TRAITS_H
#define ERMYTH_TYPE_TRAITS_H

namespace types
{
  template<typename T, T v>
  struct integral_constant
  {
    typedef T                           value_type;
    typedef integral_constant<T, v>     type;
    static T const                      value = v;
  };

  typedef integral_constant<bool, true>         true_type;
  typedef integral_constant<bool, false>        false_type;

  template<typename>
  struct is_integral
  : false_type
  {
  };

#define ADD_INTEGRAL(type)      \
  template<>                    \
  struct is_integral<type>      \
  : true_type                   \
  {                             \
  }

  ADD_INTEGRAL (         bool );
  ADD_INTEGRAL (  signed char );
  ADD_INTEGRAL (unsigned char );
  ADD_INTEGRAL (  signed short);
  ADD_INTEGRAL (unsigned short);
  ADD_INTEGRAL (  signed int  );
  ADD_INTEGRAL (unsigned int  );
  ADD_INTEGRAL (  signed long );
  ADD_INTEGRAL (unsigned long );

#undef ADD_INTEGRAL

  enum name
  {
    type_nothing,
    type_bool,
    type_char,
    type_int,
    type_charptr
  };

  typedef integral_constant<name, type_nothing> nothing_type;
  template<typename>
  struct get
  : public nothing_type
  {
  };

#define ADD_TYPE(real_type, type_name)                                          \
  typedef integral_constant<name, type_ ## type_name> type_name ## _type;       \
  template<>                                                                    \
  struct get<real_type>                                                         \
  : public type_name ## _type                                                   \
  {                                                                             \
  }

  ADD_TYPE (bool, bool);
  ADD_TYPE (char, char);
  ADD_TYPE (int, int);
  ADD_TYPE (char *, charptr);

#undef ADD_TYPE
}

#endif
