/**
 * servers.h: Data structures related to network servers.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005 William Pitcock, et al.
 * Rights to this code are as documented in doc/pod/license.pod.
 *
 * $Id: servers.h,v 1.7 2007/09/22 14:27:26 pippijn Exp $
 */

#ifndef SERVERS_H
#define SERVERS_H

#include <ermyth/callback.h>
#include <util/containers.h>

/* servers struct */
class server_t : public zero_initialised
{
  struct callbacks
  {
    functor::callback1<server_t *> eob;
  };

public:
  char *name;
  char *desc;
  char *sid;

  unsigned int hops;
  unsigned int users;
  unsigned int invis;
  unsigned int opers;
  unsigned int away;

  time_t connected_since;

  unsigned int flags;

  server_t *uplink;		/* uplink server */
  list_t children;		/* children linked to me */
  list_t userlist;		/* users attached to me */

  static callbacks callback;
};

enum server_flag
{
  SF_HIDE         = 1 << 0,
  SF_EOB          = 1 << 1,     /* Burst finished (we have all users/channels) -- jilles */
  SF_EOB2         = 1 << 2,     /* Is EOB but an uplink is not (for P10) */
  SF_JUPE_PENDING = 1 << 3      /* Sent SQUIT request, will introduce jupe when it dies (unconnect semantics) */
};

/* tld list struct */
struct tld_t : zero_initialised
{
  unsigned index;

  typedef indexing_vector<tld_t> list_type;
  static list_type list;

  char *name;
};

inline char const *
SERVER_NAME (server_t *s)
{
  return s->sid[0] ? s->sid : s->name;
}
#define ME			(ircd->uses_uid ? me.numeric : me.name)

E void init_servers (void);

E tld_t *tld_add (char const * const name);
E void tld_delete (char const * const name);
E tld_t *tld_find (char const * const name);
E void tld_cleanup ();

E server_t *server_add (char const * const name, unsigned int hops, char const * const uplink, char const * const id, char const * const desc);
E void server_delete (char const * const name);
E void server_delete (server_t *s, bool force = false);
E server_t *server_find (char const * const name);

#endif
