/**
 * predicates.h: Commonly used predicates.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are documented in COPYING.
 *
 * $Id: predicates.h,v 1.1 2007/09/22 14:27:26 pippijn Exp $
 */

#ifndef ERMYTH_PREDICATES_H
#define ERMYTH_PREDICATES_H

#include <util/exception.h>
#include <util/string.h>

namespace hash
{
  struct one_at_a_time
  {
    std::size_t operator () (char const *s) const
    {
      unsigned long hash = 0;
  
      // use the one-at-a-time hash function.
      // see http://burtleburtle.net/bob/hash/doobs.html
      while (*s)
        {
          hash += *s++;
          hash += hash << 10;
          hash ^= hash >>  6;
        }
  
      hash += hash <<  3;
      hash ^= hash >> 11;
      hash += hash << 15;
  
      return hash;
    }
  };

#if 0
  struct myhash
  {
    std::size_t operator () (char const *s) const
    {
    }
  };
#endif
}

typedef hash::one_at_a_time str_hash;

struct str_eq
{
  bool operator () (char const * const s1, char const * const s2) const
  {
    if (s1 == NULL || s2 == NULL)
      throw nullpointer_exception ();
    return !strcmp (s1, s2);
  }
};

struct str_lt
{
  bool operator () (char const * const s1, char const * const s2) const
  {
    if (s1 == NULL || s2 == NULL)
      throw nullpointer_exception ();
    return strcmp (s1, s2) < 0;
  }
};

struct strcase_eq
{
  bool operator () (char const * const s1, char const * const s2) const
  {
    if (s1 == NULL || s2 == NULL)
      throw nullpointer_exception ();
    return !strcasecmp (s1, s2);
  }
};

struct strcase_lt
{
  bool operator () (char const * const s1, char const * const s2) const
  {
    if (s1 == NULL || s2 == NULL)
      throw nullpointer_exception ();
    return strcasecmp (s1, s2) < 0;
  }
};

struct irccase_eq
{
  bool operator () (char const * const s1, char const * const s2) const
  {
    if (s1 == NULL || s2 == NULL)
      throw nullpointer_exception ();
    return !irccasecmp (s1, s2);
  }
};

struct irccase_lt
{
  bool operator () (char const * const s1, char const * const s2) const
  {
    if (s1 == NULL || s2 == NULL)
      throw nullpointer_exception ();
    return irccasecmp (s1, s2) < 0;
  }
};

#endif
