#include <svsconfig.h>
#include <libermyth.h>

#include <util/time.h>

#ifdef HAVE_GETTIMEOFDAY
/* starts a timer */
void
s_time (struct timeval *sttime)
{
  gettimeofday (sttime, NULL);
}
#endif

#ifdef HAVE_GETTIMEOFDAY
/* ends a timer */
void
e_time (struct timeval sttime, struct timeval *ttime)
{
  struct timeval now;

  gettimeofday (&now, NULL);
  timersub (&now, &sttime, ttime);
}
#endif

#ifdef HAVE_GETTIMEOFDAY
/* translates microseconds into miliseconds */
int
tv2ms (struct timeval *tv)
{
  return (tv->tv_sec * 1000) + (int) (tv->tv_usec / 1000);
}
#endif

/* return the time elapsed since an event */
char *
time_ago (time_t event)
{
  static char ret[128];
  int years, weeks, days, hours, minutes, seconds;

  event = NOW - event;
  years = weeks = days = hours = minutes = seconds = 0;

  while (event >= 60 * 60 * 24 * 365)
    {
      event -= 60 * 60 * 24 * 365;
      years++;
    }
  while (event >= 60 * 60 * 24 * 7)
    {
      event -= 60 * 60 * 24 * 7;
      weeks++;
    }
  while (event >= 60 * 60 * 24)
    {
      event -= 60 * 60 * 24;
      days++;
    }
  while (event >= 60 * 60)
    {
      event -= 60 * 60;
      hours++;
    }
  while (event >= 60)
    {
      event -= 60;
      minutes++;
    }

  seconds = event;

  if (years)
    snprintf (ret, sizeof (ret), "%d year%s, %d week%s, %d day%s, %02d:%02d:%02d", years, years == 1 ? "" : "s", weeks, weeks == 1 ? "" : "s", days, days == 1 ? "" : "s", hours, minutes, seconds);
  else if (weeks)
    snprintf (ret, sizeof (ret), "%d week%s, %d day%s, %02d:%02d:%02d", weeks, weeks == 1 ? "" : "s", days, days == 1 ? "" : "s", hours, minutes, seconds);
  else if (days)
    snprintf (ret, sizeof (ret), "%d day%s, %02d:%02d:%02d", days, days == 1 ? "" : "s", hours, minutes, seconds);
  else if (hours)
    snprintf (ret, sizeof (ret), "%d hour%s, %d minute%s, %d second%s", hours, hours == 1 ? "" : "s", minutes, minutes == 1 ? "" : "s", seconds, seconds == 1 ? "" : "s");
  else if (minutes)
    snprintf (ret, sizeof (ret), "%d minute%s, %d second%s", minutes, minutes == 1 ? "" : "s", seconds, seconds == 1 ? "" : "s");
  else
    snprintf (ret, sizeof (ret), "%d second%s", seconds, seconds == 1 ? "" : "s");

  return ret;
}

char *
timediff (time_t seconds)
{
  static char buf[BUFSIZ];
  long unsigned days, hours, minutes;

  days = seconds / 86400;
  seconds %= 86400;
  hours = seconds / 3600;
  hours %= 3600;
  minutes = seconds / 60;
  minutes %= 60;
  seconds %= 60;

  snprintf (buf, sizeof (buf), "%lu day%s, %lu:%02lu:%02lu", days, (days == 1) ? "" : "s", hours, minutes, (long unsigned) seconds);

  return buf;
}
