/**
 * ircd_noowner.C: Module to disable owner (+q) mode.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 * Module to disable owner (+q) mode.
 * This will stop Atheme setting this mode by itself, but it can still
 * be used via OperServ MODE etc.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2006 Jilles Tjoelker
 * Rights to this code are as documented in doc/pod/license.pod.
 *
 * $Id: ircd_noowner.C,v 1.5 2007/09/22 14:27:27 pippijn Exp $
 */

#include "atheme.h"
#include <ermyth/module.h>

static char const rcsid[] = "$Id: ircd_noowner.C,v 1.5 2007/09/22 14:27:27 pippijn Exp $";

REGISTER_MODULE ("ircd_noowner", false, "The Ermyth Team <http://ermyth.xinutec.org>");

static bool oldflag;

bool
_modinit (module *m)
{
  if (ircd == NULL)
    {
      slog (LG_ERROR, "Module %s must be loaded after a protocol module.", m->name);
      return false;
    }
  oldflag = ircd->uses_owner;
  ircd->uses_owner = false;
  update_chanacs_flags ();

  return true;
}

void
_moddeinit ()
{
  ircd->uses_owner = oldflag;
  update_chanacs_flags ();
}
