/**
 * logout.C: This file contains code for the ChanServ LOGOUT functions.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005 William Pitcock, et al.
 * Rights to this code are as documented in doc/pod/license.pod.
 *
 * $Id: logout.C,v 1.8 2007/09/22 14:27:28 pippijn Exp $
 */

#include "atheme.h"
#include <libermyth.h>
#include <ermyth/module.h>
#include <account/myuser.h>

static char const rcsid[] = "$Id: logout.C,v 1.8 2007/09/22 14:27:28 pippijn Exp $";

REGISTER_MODULE ("nickserv/logout", false, "The Ermyth Team <http://ermyth.xinutec.org>");

static void ns_cmd_logout (sourceinfo_t *si, int parc, char *parv[]);

command_t const ns_logout = { "LOGOUT", N_("Logs your services session out."), AC_NONE, 2, ns_cmd_logout };

E cmdvec ns_cmdtree;
E helpvec ns_helptree;

bool
_modinit (module *m)
{
  ns_cmdtree << ns_logout;
  help_addentry (ns_helptree, "LOGOUT", "help/nickserv/logout", NULL);

  return true;
}

void
_moddeinit ()
{
  ns_cmdtree >> ns_logout;
  help_delentry (ns_helptree, "LOGOUT");
}

static void
ns_cmd_logout (sourceinfo_t *si, int parc, char *parv[])
{
  user_t *u = si->su;
  char *user = parv[0];
  char *pass = parv[1];

  if ((!si->smu) && (!user || !pass))
    {
      command_fail (si, fault::nochange, _("You are not logged in."));
      return;
    }

  if (user && pass)
    {
      myuser_t *mu = myuser_t::find (user);

      if (!mu)
	{
	  command_fail (si, fault::nosuch_target, _("\2%s\2 is not registered."), user);
	  return;
	}

#if 0
      if ((mu->verify_password (pass)) && (mu->user))
	{
	  u = mu->user;
	  notice (nicksvs.nick, u->nick, "You were logged out by \2%s\2.", si->su->nick);
	}
      else
	{
	  command_fail (si, fault::authfail, _("Authentication failed. Invalid password for \2%s\2."), mu->name);
	  return;
	}
#endif
      /* remove this for now -- jilles */
      command_fail (si, fault::unimplemented, _("External logout is not yet implemented."));
      return;
    }
  else if (si->su == NULL)
    {
      command_fail (si, fault::needmoreparams, STR_INSUFFICIENT_PARAMS, "LOGOUT");
      command_fail (si, fault::needmoreparams, _("Syntax: LOGOUT <target> <password>"));
      return;
    }


  if (is_soper (si->smu))
    snoop ("DESOPER: \2%s\2 as \2%s\2", si->su->nick, si->smu->name);

  if (si->su != u)
    {
      logcommand (si, CMDLOG_LOGIN, "LOGOUT %s", u->nick);
      command_success_nodata (si, _("\2%s\2 has been logged out."), si->su->nick);
    }
  else
    {
      logcommand (si, CMDLOG_LOGIN, "LOGOUT");
      command_success_nodata (si, _("You have been logged out."));
    }

  si->smu->lastlogin = NOW;
  if (!phandler->ircd_on_logout (si->su->nick, si->smu->name, NULL))
    {
      si->smu->logins.erase (si->su);
      si->su->myuser = NULL;
    }
}
