/**
 * jupe.C: Jupiters a server.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005-2006 William Pitcock, et al.
 * Rights to this code are as documented in doc/pod/license.pod.
 *
 * $Id: jupe.C,v 1.8 2007/09/22 14:27:28 pippijn Exp $
 */

#include "atheme.h"
#include <ermyth/module.h>

static char const rcsid[] = "$Id: jupe.C,v 1.8 2007/09/22 14:27:28 pippijn Exp $";

REGISTER_MODULE ("operserv/jupe", false, "The Ermyth Team <http://ermyth.xinutec.org>");

static void os_cmd_jupe (sourceinfo_t *si, int parc, char *parv[]);

command_t const os_jupe = { "JUPE", N_("Jupiters a server."), PRIV_JUPE, 2, os_cmd_jupe };

E cmdvec os_cmdtree;
E helpvec os_helptree;

bool
_modinit (module *m)
{
  os_cmdtree << os_jupe;
  help_addentry (os_helptree, "JUPE", "help/operserv/jupe", NULL);

  return true;
}

void
_moddeinit ()
{
  os_cmdtree >> os_jupe;
  help_delentry (os_helptree, "JUPE");
}

static void
os_cmd_jupe (sourceinfo_t *si, int parc, char *parv[])
{
  char *server = parv[0];
  char *reason = parv[1];
  char reasonbuf[BUFSIZE];

  if (!server || !reason)
    {
      command_fail (si, fault::needmoreparams, STR_INSUFFICIENT_PARAMS, "JUPE");
      command_fail (si, fault::needmoreparams, _("Usage: JUPE <server> <reason>"));
      return;
    }

  if (!strchr (server, '.'))
    {
      command_fail (si, fault::badparams, _("\2%s\2 is not a valid server name."), server);
      return;
    }

  if (!irccasecmp (server, me.name))
    {
      command_fail (si, fault::noprivs, _("\2%s\2 is the services server; it cannot be jupitered!"), server);
      return;
    }

  if (!irccasecmp (server, me.actual))
    {
      command_fail (si, fault::noprivs, _("\2%s\2 is the current uplink; it cannot be jupitered!"), server);
      return;
    }

  logcommand (si, CMDLOG_ADMIN, "JUPE %s %s", server, reason);

  snprintf (reasonbuf, BUFSIZE, "[%s] %s", get_oper_name (si), reason);
  phandler->jupe (server, reasonbuf);

  command_success_nodata (si, _("\2%s\2 has been jupitered."), server);
}
