/**
 * main.C: This file contains the main() routine.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005-2006 Atheme Development Group
 * Rights to this code are documented in doc/pod/license.pod.
 *
 * $Id: main.C,v 1.8 2007/09/22 14:27:28 pippijn Exp $
 */

#include "atheme.h"
#include "confparse.h"
#include <ermyth/module.h>

static char const rcsid[] = "$Id: main.C,v 1.8 2007/09/22 14:27:28 pippijn Exp $";

REGISTER_MODULE ("operserv/main", false, "The Ermyth Team <http://ermyth.xinutec.org>");

static void on_config_ready (void);

cmdvec os_cmdtree;
helpvec os_helptree;

/* main services client routine */
static void
operserv (sourceinfo_t *si, int parc, char *parv[])
{
  char *cmd;
  char *text;
  char orig[BUFSIZE];

  /* this should never happen */
  if (parv[0][0] == '&')
    {
      slog (LG_ERROR, "services(): got parv with local channel: %s", parv[0]);
      return;
    }

  /* make a copy of the original for debugging */
  strlcpy (orig, parv[parc - 1], BUFSIZE);

  /* lets go through this to get the command */
  cmd = strtok (parv[parc - 1], " ");
  text = strtok (NULL, "");

  if (!cmd)
    return;
  if (*cmd == '\001')
    {
      handle_ctcp_common (si, cmd, text);
      return;
    }

  /* take the command through the hash table */
  command_exec_split (si->service, si, cmd, text, os_cmdtree);
}

static void
on_config_ready (void)
{
  if (opersvs.me)
    del_service (opersvs.me);

  opersvs.me = add_service (opersvs.nick, opersvs.user, opersvs.host, opersvs.real, operserv, os_cmdtree);
  opersvs.disp = opersvs.me->disp;
}

bool
_modinit (module *m)
{
  ConfTable::callback.ready.attach (on_config_ready);

  if (!cold_start)
    {
      opersvs.me = add_service (opersvs.nick, opersvs.user, opersvs.host, opersvs.real, operserv, os_cmdtree);
      opersvs.disp = opersvs.me->disp;
    }

  return true;
}

void
_moddeinit ()
{
  if (opersvs.me)
    {
      del_service (opersvs.me);
      opersvs.me = NULL;
    }

  ConfTable::callback.ready.detach (on_config_ready);
}
