package Ermyth;

use strict;
use warnings FATAL => 'all';
use utf8;

my $rcsid = '$Id: Ermyth.pm,v 1.4 2007/07/28 17:26:27 pippijn Exp $';

use Carp;
use JSON::RPC::Client;
use Ermyth::Accessor;

acc_rw "authcookie";
acc_rw "ip";
acc_rw "user";

our $VERSION = "1.02";

=head1 Ermyth

A Perl JSON Remote Procedure Call interface for Ermyth.

=cut

sub new {
   my ($class, $args) = @_;
   my $self = bless {
      rpc => new JSON::RPC::Client,
      uri => delete $args->{uri} || croak "You need to provide an URI",
      %$args
   }, $class;

   $self
}

sub login {
   my ($self, $user, $password) = @_;
   
   my $obj = {
      method => "ermyth.login",
      params => [$user, $password, $self->ip],
   };

   my $res = $self->{rpc}->call ($self->{uri}, $obj);

   if ($res) {
      return Ermyth::Error->new ($res)
         if $res->is_error;
      $self->authcookie ($res->result->{result});
      $self->user       ($user);
   } else {
      return Ermyth::HTTPError->new ($self->{rpc}->status_line);
   }

   $self->authcookie
}

sub logout {
   my ($self) = @_;
   
   my $obj = {
      method => "ermyth.logout",
      params => [$self->authcookie, $self->user],
   };

   my $res = $self->{rpc}->call ($self->{uri}, $obj);

   if ($res) {
      return Ermyth::Error->new ($res)
         if $res->is_error;
      $self->authcookie (undef);
      $self->user       (undef);
      $self->ip         (undef);
   } else {
      return Ermyth::HTTPError->new ($self->{rpc}->status_line);
   }

   1
}

sub command {
   my ($self, $svs, @args) = @_;

   my $obj = {
      method => "ermyth.command",
      params => [$self->authcookie, $self->user, $self->ip, $svs->name, @args],
   };

   my $res = $self->{rpc}->call ($self->{uri}, $obj);

   if ($res) {
      return Ermyth::Error->new ($res)
         if $res->is_error;
      return $res->result->{result}
         if $res->result;
      return;
   } else {
      return Ermyth::HTTPError->new ($self->{rpc}->status_line);
   }
}

regservice "ChanServ";
regservice "GameServ";
regservice "MemoServ";
regservice "NickServ";
regservice "OperServ";


package Ermyth::Error;

use strict;
use warnings;
use utf8;

use Ermyth::Accessor;

acc_ro "message";
acc_ro "code";

sub new {
   my ($class, $args) = @_;

   bless {
      message => $args->error_message,
      code    => $args->{content}->{error}->{code},
   }, $class
}


package Ermyth::HTTPError;

use strict;
use warnings;
use utf8;

use Ermyth::Accessor;

acc_ro "message";
acc_ro "code";

sub new {
   my ($class, $error) = @_;

   my ($code, $text) = $error =~ /(\d+) (.+)/;

   bless {
      message => $text,
      code    => $code,
   }, $class
}

=head1 AUTHOR

Copyright E<xa9> 2007 Pippijn van Steenhoven

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1
