package Ermyth::Web;

use strict;
use warnings FATAL => 'all';
use utf8;

my $rcsid = '$Id: Web.pm,v 1.4 2007/07/28 17:26:27 pippijn Exp $';

use Catalyst::Runtime '5.70';

# Set flags and add plugins for the application
#
#         -Debug: activates the debug mode for very useful log messages
#   ConfigLoader: will load the configuration from a YAML file in the
#                 application's home directory

use Catalyst qw/
   -Debug
   ConfigLoader
   Static::Simple
   Session
   Session::State::Cookie
   Session::Store::File
   /;

our $VERSION = '0.01';

# Configure the application. 
#
# Note that settings in ermyth_web.yml (or other external
# configuration file that you set up manually) take precedence
# over this when using ConfigLoader. Thus configuration
# details given here can function as a default configuration,
# with a external configuration file acting as an override for
# local deployment.

__PACKAGE__->config (
   name => 'Ermyth Web',
   base => '',
   'Model::Pod' => {
      root_dir => __PACKAGE__->path_to ('root/pod'),
   },
   'Model::Ermyth' => {
      uri => "http://localhost:8080/jsonrpc",
   },
);

# Start the application
__PACKAGE__->setup;


=head1 NAME

Ermyth::Web - Catalyst based web frontend for Ermyth.

=head1 SYNOPSIS

    script/ermyth_web_server.pl

In lighttpd:

   fastcgi.server = (
      "/ermyth" => (
         "Ermyth" => (
            "socket"       => "/tmp/ermyth.socket",
            "check-local"  => "disable",
            "bin-path"     => "/home/pippijn/code/ermyth/wwwroot/script/ermyth_web_fastcgi.pl",
            "min-procs"    => 1,
            "max-procs"    => 1,
            "idle-timeout" => 3600 # Yes, this long, because every time a new
               # server is spawned, Ermyth has to look up available commands
               # in Services.
            )
         )
      )

=head1 DESCRIPTION

This provides a web frontend for Ermyth IRC Services.

=head1 SEE ALSO

L<Ermyth::Web::Controller::Root>, L<Catalyst>, L<Ermyth>

=head1 AUTHOR

Copyright E<xa9> 2007 Pippijn van Steenhoven

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
