package Ermyth::Web::Controller::Account;

use strict;
#use warnings FATAL => 'all';
use utf8;

my $rcsid = '$Id: Account.pm,v 1.4 2007/07/28 17:26:27 pippijn Exp $';

use base 'Catalyst::Controller::FormBuilder';

__PACKAGE__->config (
   'Controller::FormBuilder' => {
      new => {
         method   => 'post',
         validate => {
            passagain => {
               javascript => '!= form.elements["pass"].value',
               perl       => 'eq $form->field ("pass")',
            },
         },
         debug    => 0,
      },
      obj_name => 'fb',
   },
);

sub invalidate {
   my ($self, $field, $form, $res) = @_;
   $form->field (
      name     => $field,
      message  => $res->message,
      validate => { perl => 'eq 0' },
   );
   $form->validate;
}

sub default : Private {
   my ($self, $c) = @_;

   $c->stash->{current} = "/account";
   $c->stash->{pod} = $c->model('Pod')->html ("account.pod");
   $c->stash->{template} = 'static.tt'
}


sub login : Local Form {
   my ($self, $c) = @_;
   my $ermyth = $c->model('Ermyth')->instance ($c);
   $c->stash->{current} = "/account/login";

   my $form = $self->formbuilder;

   return $c->forward ("/account/logout")
      if $c->session->{authcookie};

   if ($form->submitted && $form->validate) {
      my $fields = $form->fields;
      my ($name, $pass) = ($fields->{nick}, $fields->{pass});
      my $res = $ermyth->login ($name, $pass);

      if (ref $res eq "Ermyth::HTTPError") {
         $c->stash->{error}    = $res;
         $c->stash->{template} = 'lib/httperror.tt';
         return;
      } elsif (ref $res eq "Ermyth::Error") {
         if ($res->code == 5) {
            $self->invalidate ('pass', $form, $res);
         } elsif ($res->code == 3) {
            $self->invalidate ('nick', $form, $res);
         } else {
            $c->stash->{error} = "Unknown error: code (" . $res->code . "), message (" . $res->message . ")"; 
         }

         $c->stash->{template} = 'lib/form.tt';
         return;
      }

      $c->session->{authcookie} = $res;
      $c->session->{account}    = $name;

      return $c->forward ("/home");
   }

   $c->stash->{template} = 'lib/form.tt'
}

sub logout : Local {
   my ($self, $c) = @_;
   my $ermyth = $c->model('Ermyth')->instance ($c);
   $c->stash->{current} = "/account/logout";

   # call ermyth
   my $res = $ermyth->logout;

   # invalidate session
   $c->session->{authcookie} = undef;
   $c->session->{account}    = undef;
   
   if (ref $res eq "Ermyth::HTTPError") {
      $c->stash->{error}    = $res;
      $c->stash->{template} = 'lib/httperror.tt';
      
      return;
   } elsif (ref $res eq "Ermyth::Error") {
      if ($res->code == 5) {
         $c->stash->{error} = "You could not be logged out because: " . $res->message
            . " This likely means that you tried to log out twice or your session timed out.";
      } elsif ($res->code == 3) {
         $c->stash->{error} = "You could not be logged out because: " . $res->message
            . " This likely means that you tried to log out twice or never logged in"
            . " (do you have cookies allowed for this site?).";
      } else {
         $c->stash->{error} = "Unknown error: code (" . $res->code . "), message (" . $res->message . ")"; 
      }
      
      $c->stash->{pod} = $c->model('Pod')->html ("account/logout.pod", 'error');
      $c->stash->{template} = 'account/logout.tt';
      return;
   }

   $c->stash->{pod} = $c->model('Pod')->html ("account/logout.pod");
   $c->stash->{template} = 'account/logout.tt'
}

sub register : Local Form {
   my ($self, $c) = @_;
   my $ermyth = $c->model('Ermyth')->instance ($c);
   $c->stash->{current} = "/account/register";

   my $form = $self->formbuilder;

   if ($form->submitted && $form->validate) {
      my $fields = $form->fields;
      my ($name, $pass, $email) = ($fields->{nick}, $fields->{pass}, $fields->{email});
      my $res = $ermyth->nickserv->register ($name, $pass, $email);

      if (ref $res eq "Ermyth::HTTPError") {
         $c->stash->{error}    = $res;
         $c->stash->{template} = 'lib/httperror.tt';

         return;
      } elsif (ref $res eq "Ermyth::Error") {
         if ($res->code == 8) {
            $self->invalidate ('nick', $form, $res);
         } elsif ($res->code == 2) {
            $self->invalidate ('pass', $form, $res);
         } else {
            $c->stash->{error} = "Unknown error: code (" . $res->code . "), message (" . $res->message . ")"; 
         }

         $c->stash->{template} = 'lib/form.tt';
         return;
      }

      $c->stash->{pod}   = $c->model('Pod')->html ("account/register.pod");
      $c->stash->{nick}  = $name;
      $c->stash->{pass}  = $pass;
      $c->stash->{email} = $email;
      $c->stash->{template} = 'account/register.tt';
      return;
   }

   $c->stash->{template} = 'lib/form.tt'
}

sub profile : Local Form {
   my ($self, $c) = @_;
   my $ermyth = $c->model('Ermyth')->instance ($c);
   $c->stash->{current} = "/account/profile";
   $c->stash->{ucfirst} = sub { $_[0] =~ s/_/ /g; ucfirst $_[0] };
   $c->stash->{encode} = sub { utf8::encode ($_[0]); $_[0] };

   my $form = $self->formbuilder;
   my $fields = $form->fields;

   my $set = sub {
      my ($c, $fields, $ermyth, $property) = @_;
      my $error = 0;
      if ($fields->{$property}) {
         my $value = $fields->{$property};
         utf8::decode ($value);
         my $res = $ermyth->nickserv->set ("PROPERTY", "$property " . $value);
         
         $c->stash->{profile}->{$property} = $value;
         
         if (ref $res eq "Ermyth::HTTPError") {
            $c->stash->{error}    = $res;
            $c->stash->{template} = 'lib/httperror.tt';
            
            $error = -500;
         } elsif (ref $res eq "Ermyth::Error") {
#            if ($res->code == 12) {
#               delete $c->stash->{$property};
#            } else {
               $c->stash->{error} = "Unknown error: code (" . $res->code . "), message (" . $res->message . ")"; 
               $error++;
#            }
         }

         $c->stash->{template} = 'lib/form.tt'
      }
      $error
   };
   
   my $res = $ermyth->nickserv->taxonomy ($c->session->{account});
   
   if (ref $res eq "Ermyth::HTTPError") {
      $c->stash->{error}    = $res;
      $c->stash->{template} = 'lib/httperror.tt';
      return
   } elsif (ref $res eq "Ermyth::Error") {
#      if ($res->code == 12) {
#         delete $c->stash->{$property};
#      } else {
         $c->stash->{error} = "Unknown error: code (" . $res->code . "), message (" . $res->message . ")"; 
#      }

      $c->stash->{pod} = $c->model('Pod')->html ("account/profile.pod", 'form');
      $c->stash->{template} = 'lib/form.tt';
      return
   }

   my @taxonomy = split /\n/, $res;
   shift @taxonomy; pop @taxonomy; # get rid of description
   for (@taxonomy) {
      my ($key, $value) = split /: /, $_, 2;
      $key =~ s/\s+$//;
      utf8::encode ($value);
      if (grep { $_ eq $key } keys %{ $form->fields }) {
         $c->stash->{profile}->{$key} = $value;
         $form->field (
            name  => $key,
            value => $value,
         );
      }
   }

   if ($form->submitted && $form->validate) {
      my $error = 0;
      $error += $set->($c, $fields, $ermyth, $_)
         for keys %$fields;

      return if $error < 0;
      $c->stash->{template} = 'account/profile.tt';
      return unless $error;
   }
   
   $c->stash->{pod} = $c->model('Pod')->html ("account/profile.pod", 'form');
   $c->stash->{template} = 'lib/form.tt'
}

sub maintainance : Local Form {
   my ($self, $c) = @_;
   my $ermyth = $c->model('Ermyth')->instance ($c);
   $c->stash->{current} = "/account/maintainance";

   my $set = sub {
      my ($c, $form, $ermyth, $subcmd) = @_;
      my $fields = $form->fields;
      my $error = 0;
      if ($fields->{$subcmd}) {
         my $res = $ermyth->nickserv->set (uc $subcmd, $fields->{$subcmd});
         
         $c->stash->{$subcmd} = $fields->{$subcmd};
         
         if (ref $res eq "Ermyth::HTTPError") {
            $c->stash->{error}    = $res;
            $c->stash->{template} = 'lib/httperror.tt';
            
            $error = -10 ** 16;
         } elsif (ref $res eq "Ermyth::Error") {
            if ($res->code == 12) {
               # The $subcmd flag is already set for <user>
               delete $c->stash->{$subcmd};
            } else {
               $c->stash->{error} = "Unknown error: code (" . $res->code . "), message (" . $res->message . ")"; 
               $error++;
            }
         }

         $c->stash->{template} = 'lib/form.tt'
      }
      $error
   };

   my $form = $self->formbuilder;

   if ($form->submitted && $form->validate) {
      my $error = 0;
      $error += $set->($c, $form, $ermyth, 'emailmemos');
      $error += $set->($c, $form, $ermyth, 'enforce');
      $error += $set->($c, $form, $ermyth, 'neverop');
      $error += $set->($c, $form, $ermyth, 'hidemail');
      $error += $set->($c, $form, $ermyth, 'noop');
      $error += $set->($c, $form, $ermyth, 'nomemo');
      $error += $set->($c, $form, $ermyth, 'email');
      $error += $set->($c, $form, $ermyth, 'password');

      if ($form->fields->{drop}) {
         my $res = $ermyth->nickserv->drop ($c->session->{account}, $form->fields->{drop});

         if (ref $res eq "Ermyth::HTTPError") {
            $c->stash->{error}    = $res;
            $c->stash->{template} = 'lib/httperror.tt';
            
            return;
         } elsif (ref $res eq "Ermyth::Error") {
            if ($res->code == 5) {
               $self->invalidate ('drop', $form, $res);
               $error++;
            } else {
               $self->invalidate ('drop', $form, $res);
               $error++;
            }
         }

         # invalidate session
         if (!$error) {
            $c->session->{authcookie} = undef;
            $c->session->{account}    = undef;
         }
      }
      
      return if $error < 0;
      $c->stash->{password} = "*" x length $form->fields->{passagain}
         if $form->fields->{passagain};
      $c->stash->{template} = 'account/maintainance.tt';
      return unless $error;
   }
   
   $c->stash->{pod} = $c->model('Pod')->html ("account/maintainance.pod", 'form');
   $c->stash->{template} = 'lib/form.tt'
}


=head1 AUTHOR

Copyright © 2007 Pippijn van Steenhoven

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1
