package Ermyth::Web::Controller::Memo;

use strict;
use warnings FATAL => 'all';
use utf8;

my $rcsid = '$Id: Memo.pm,v 1.1 2007/08/28 17:18:26 pippijn Exp $';

use base 'Catalyst::Controller::FormBuilder';

__PACKAGE__->config (
   'Controller::FormBuilder' => {
      new => {
         method => 'post',
         debug  => 0,
      },
      obj_name => 'fb',
   },
);

sub invalidate {
   my ($self, $field, $form, $res) = @_;
   $form->field (
      name     => $field,
      message  => $res->message,
      validate => { perl => 'eq 0' },
   );
   $form->validate;
}

sub default : Private {
   my ($self, $c) = @_;

   $c->stash->{current} = "/memo";
   $c->stash->{pod} = $c->model('Pod')->html ("memo.pod");
   $c->stash->{template} = 'static.tt'
}


sub delete : Local {
   my ($self, $c) = @_;
   my $ermyth = $c->model('Ermyth')->instance ($c);
   $c->stash->{current} = "/memo/delete";

   my $index = $c->req->query_parameters->{index};
   unless ($index) {
      $c->stash->{pod} = $c->model('Pod')->html ("memo/delete.pod");
      $c->stash->{template} = 'static.tt';
      return
   }

   my $res = $ermyth->memoserv->delete ($index);
   
   if (ref $res eq "Ermyth::HTTPError") {
      $c->stash->{error}    = $res;
      $c->stash->{template} = 'lib/httperror.tt';
      return;
   } elsif (ref $res eq "Ermyth::Error") {
      if ($res->code == 7) {
         $c->stash->{message} = $res->message;
#      } elsif ($res->code == 3) {
#         $self->invalidate ('nick', $form, $res);
      } else {
         $c->stash->{error} = $res;
         $c->stash->{template} = 'lib/perlerror.tt';
         return
      }

      $c->stash->{pod} = $c->model('Pod')->html ("memo/delete.pod", 'error');
      $c->stash->{template} = 'memo/delete.tt';
      return
   }

   $c->stash->{message} = $res;
   $c->stash->{template} = 'memo/delete.tt'
}

sub list : Local {
   my ($self, $c) = @_;
   my $ermyth = $c->model('Ermyth')->instance ($c);
   $c->stash->{current} = "/memo/list";

   my $res = $ermyth->memoserv->list;
   
   if (ref $res eq "Ermyth::HTTPError") {
      $c->stash->{error}    = $res;
      $c->stash->{template} = 'lib/httperror.tt';
      return;
   } elsif (ref $res eq "Ermyth::Error") {
#      if ($res->code == 5) {
#         $self->invalidate ('pass', $form, $res);
#      } elsif ($res->code == 3) {
#         $self->invalidate ('nick', $form, $res);
#      } else {
         $c->stash->{error} = $res;
#      }

      $c->stash->{template} = 'lib/perlerror.tt';
      return
   }

   my @memos = split /\n/, $res;
   my ($nrof, $new) = shift (@memos) =~ /You have (\d+) memos? \((-?\d+) new\)./; # XXX: BROKEN ERMYTH for '-'
   $c->stash->{memocount} = {
      count => $nrof,
      new   => $new,
   };

   shift @memos; # get rid of empty line
   for (@memos) {
      my ($index, $from, $sent, $status) = $_ =~ /- (\d+) From: ([^ ]+) Sent: ([^[]+) ?(.*)/;
      push @{ $c->stash->{memos} }, {
         from   => $from,
         sent   => $sent,
         status => $status,
         index  => $index,
      };
   }

   $c->stash->{template} = 'memo/list.tt'
}

sub read : Local {
   my ($self, $c) = @_;
   my $ermyth = $c->model('Ermyth')->instance ($c);
   $c->stash->{current} = "/memo/read";

   if (my $action = $c->req->query_parameters->{action}) {
      $c->forward ("/memo/delete")
         if $action eq "Delete";
      return
   }
   
   my $index = $c->req->query_parameters->{index};
   unless ($index) {
      $c->stash->{pod} = $c->model('Pod')->html ("memo/read.pod");
      $c->stash->{template} = 'static.tt';
      return
   }

   my $res = $ermyth->memoserv->read ($index);
   
   if (ref $res eq "Ermyth::HTTPError") {
      $c->stash->{error}    = $res;
      $c->stash->{template} = 'lib/httperror.tt';
      return;
   } elsif (ref $res eq "Ermyth::Error") {
      if ($res->code == 7) {
         $c->stash->{message} = $res->message;
#      } elsif ($res->code == 3) {
#         $self->invalidate ('nick', $form, $res);
      } else {
         $c->stash->{error} = $res;
         $c->stash->{template} = 'lib/perlerror.tt';
         return
      }

      $c->stash->{pod} = $c->model('Pod')->html ("memo/read.pod", 'error_nomemos');
      $c->stash->{template} = 'static.tt';
      return
   }

   my ($info, undef, $memo) = split /\n/, $res, 3;
   $memo =~ s/\\n/\n/g;
   $memo =~ s/\\r/\r/g;
   my ($from, $sent) = $info =~ /Memo \d - Sent by ([^,]+), (.+)/;
   $c->stash->{memo}     = {
      index => $index,
      from  => $from,
      sent  => $sent,
      memo  => $memo,
   };

   $c->stash->{template} = 'memo/read.tt'
}

sub send : Local Form {
   my ($self, $c) = @_;
   my $ermyth = $c->model('Ermyth')->instance ($c);
   $c->stash->{current} = "/memo/send";

   my $form = $self->formbuilder;

   if ($form->submitted && $form->validate) {
      my $fields = $form->fields;
      my ($target, $memo) = ($fields->{target}, $fields->{memo});
      $memo =~ s/\n/\\n/g;
      $memo =~ s/\r/\\r/g;
      utf8::decode $memo;
      my $res = $ermyth->memoserv->send ($target, $memo);

      if (ref $res eq "Ermyth::HTTPError") {
         $c->stash->{error}    = $res;
         $c->stash->{template} = 'lib/httperror.tt';
         return;
      } elsif (ref $res eq "Ermyth::Error") {
#         if ($res->code == 5) {
#            $self->invalidate ('pass', $form, $res);
#         } elsif ($res->code == 3) {
#            $self->invalidate ('nick', $form, $res);
#         } else {
            $c->stash->{error} = "Unknown error: code (" . $res->code . "), message (" . $res->message . ")"; 
#         }

         $c->stash->{template} = 'lib/form.tt';
         return;
      }

      $c->stash->{message}  = $res;
      $c->stash->{template} = 'memo/send.tt';
      return
   }

   $c->stash->{template} = 'lib/form.tt'
}

=head1 AUTHOR

Copyright © 2007 Pippijn van Steenhoven

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1
