package Ermyth::Web::Model::Ermyth;

use strict;
use warnings FATAL => 'all';
use utf8;

my $rcsid = '$Id: Ermyth.pm,v 1.4 2007/07/28 17:26:27 pippijn Exp $';

use base qw/Catalyst::Model/;
use NEXT;
use Carp;

use Ermyth;

our $VERSION = 0.01;

=head1 NAME

Ermyth::Web::Model::Ermyth - Catalyst Model wrapper for the Ermyth class

=head1 SYNOPSIS

   # in your application configuration (where you 'use Catalyst'):
   __PACKAGE__->config (
      'Model::Ermyth' => {
         uri => 'http://localhost:8080/jsonrpc',
      },
   );

   # in your Controller:
   sub login : Local {
      my ($self, $c) = @_;
      my $ermyth = $c->model('Ermyth')->instance ($c);

      my $res = $ermyth->login ("myaccount", "mypass");

      # do checks like you would with native Ermyth
      # do something with the authcookie. you could put it into a session cookie
   }

=head1 DESCRIPTION

This is the 'Ermyth' model class.

=head1 METHODS

=over

=item new

Initialises the Ermyth RPC class.

=cut

sub new {
   my $self = shift->NEXT::new (@_);
   
   croak "->config->{uri} must be defined for this model\n"
      unless $self->{uri};
   
   $self->{ermyth} = new Ermyth { uri => $self->{uri} };
   
   $self
}

=item $self->instance ($c)

Returns the current Ermyth object with authcookie, user and ip gained from $c.

=cut

sub instance {
   my ($self, $c) = @_;

   $self->{ermyth}->authcookie ($c->session->{authcookie} || '.');
   $self->{ermyth}->user       ($c->session->{account} || '.');
   $self->{ermyth}->ip         ($c->req->address || 'localhost');

   $self->{ermyth}
}

=back

=head1 SEE ALSO

L<Ermyth>

=head1 AUTHOR

Copyright © 2007 Pippijn van Steenhoven

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1
