/**
 * symbols.h: Symbol table. This is a Proof-Of-Concept and is not used anywhere yet. It may never get any use.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in doc/pod/gplicense.pod.
 *
 * $Id: symbols.h,v 1.3 2007/09/16 18:54:42 pippijn Exp $
 */

#ifndef ERMYTH_SYMBOL_H
#define ERMYTH_SYMBOL_H

#include <boost/any.hpp>

#include <map>
#include <string>
#include <stdexcept>

class symbol_table
{
  typedef std::map<std::string const, boost::any> map_type;
  map_type symbol_map;

public:
  template<typename T>
  void insert (std::string const &name, T symbol)
  {
    try
      {
        symbol_map.at (name);
        throw std::runtime_error ("There is already a symbol with name " + name + " in the table");
      }
    catch (std::out_of_range &e)
      {
        boost::any sym = symbol;
        symbol_map[name] = sym;
      }
  }

  template<typename T>
  T retrieve (std::string const &name)
  {
    try
      {
        boost::any sym = symbol_map.at (name);
        return boost::any_cast<T> (sym);
      }
    catch (std::out_of_range &)
      {
        throw std::runtime_error ("Symbol table does not contain symbol " + name);
      }
    catch (boost::bad_any_cast &)
      {
        throw std::bad_cast ();
      }
  }
};

extern symbol_table symbols;

#endif
